'From Smalltalk-80, Version 2.2 of July 4, 1987 on 9 July 1987 at 11:27:15 am'!

MouseMenuController subclass: #FinancialHistoryController
	instanceVariableNames: ''
	classVariableNames: 'FHYellowButtonMenu FHYellowButtonMessages '
	poolDictionaries: ''
	category: 'Demo-FinancialTools'!


!FinancialHistoryController methodsFor: 'initialize-release'!

initialize
	super initialize.
	self initializeYellowButtonMenu! !

!FinancialHistoryController methodsFor: 'menu messages'!

receive
	"Ask what amount is being spent on what."

	| spendOn amount |
	spendOn _ FillInTheBlank request: 'Receive from what?'.
	spendOn isEmpty ifTrue: [^self].
	amount  _ FillInTheBlank request: 'How much from ', spendOn,' ?'.
	amount isEmpty ifTrue: [^self].
	amount _ Number readFrom: (ReadStream on: amount).
	model receive: amount from: spendOn.!

spend
	"Ask what amount is being spent on what."

	| spendOn amount |
	spendOn _ FillInTheBlank request: 'Spend for what?'.
	spendOn isEmpty ifTrue: [^self].
	amount  _ FillInTheBlank request: 'How much for ', spendOn,' ?'.
	amount isEmpty ifTrue: [^self].
	amount _ Number readFrom: (ReadStream on: amount).
	model spend: amount for: spendOn.! !

!FinancialHistoryController methodsFor: 'control defaults'!

isControlActive
	^super isControlActive & sensor blueButtonPressed not! !

!FinancialHistoryController methodsFor: 'private'!

initializeYellowButtonMenu 
	self yellowButtonMenu: FHYellowButtonMenu 
		yellowButtonMessages: FHYellowButtonMessages! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FinancialHistoryController class
	instanceVariableNames: ''!


!FinancialHistoryController class methodsFor: 'class initialization'!

initialize
	"Specify the yellow button menu items and actions."

	FHYellowButtonMenu _
		PopUpMenu
			labels: 
'spend
receive'.

	FHYellowButtonMessages _ #(spend receive).

	"FinancialHistoryController initialize."! !

FinancialHistoryController initialize!


Object subclass: #FinancialHistory
	instanceVariableNames: 'cashOnHand incomes expenditures '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Demo-FinancialTools'!


!FinancialHistory methodsFor: 'transactions'!

receive: amount from: source
	"Receive amount of money from source and increment cash on hand by this amount."

	incomes at: source
			put: (self totalReceivedFrom: source) + amount.
	cashOnHand _ cashOnHand + amount.
	incomes changed!

spend: amount for: reason
	"Spend amount of money for reason and decrement cash on hand by this amount."

	expenditures at: reason
				 put: (self totalSpentFor: reason) + amount.
	cashOnHand _ cashOnHand - amount.
	expenditures changed! !

!FinancialHistory methodsFor: 'inquiries'!

cashOnHand
	"What is the current balance?"

	^cashOnHand!

expenditures
	^expenditures!

incomes
	^incomes!

totalReceivedFrom: source
	(incomes includesKey: source)
		ifTrue: [^incomes at: source]
		ifFalse: [^0]!

totalSpentFor: reason
	(expenditures includesKey: reason)
		ifTrue: [^expenditures at: reason]
		ifFalse: [^0]! !

!FinancialHistory methodsFor: 'private'!

setInitialBalance: amount
	cashOnHand _ amount.
	incomes _ Dictionary new.
	expenditures _ Dictionary new! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FinancialHistory class
	instanceVariableNames: ''!


!FinancialHistory class methodsFor: 'instance creation'!

initialBalance: amount
	"Create a FinancialHistory with amount as the initial balance."
	
	^super new setInitialBalance: amount!

new
	"Create a FinancialHistory with 0 as the initial balance."
	
	^super new setInitialBalance: 0! !

!FinancialHistory class methodsFor: 'examples'!

exampleWorkspace
	"Execute the expressions in this workspace in order to examine the behavior of the FinancialHistory bar charts."

				"do this first"
		Smalltalk at: #HouseholdFinances put: nil.

				"then these"
	HouseholdFinances _ FinancialHistory initialBalance: 1560.
	HouseholdFinances spend: 700 for: 'rent'.
	HouseholdFinances spend: 78.53 for: 'food'.
	HouseholdFinances receive: 820 from: 'pay'.
	HouseholdFinances receive: 22.15 from: 'interest'.
	HouseholdFinances spend: 135.65 for: 'utils'.
	HouseholdFinances spend: 146.14 for: 'food'.

	FinancialHistoryView open: HouseholdFinances! !

View subclass: #BarChartView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Demo-FinancialTools'!


!BarChartView methodsFor: 'accessing'!

barFrame
	^self insetDisplayBox insetBy: (50 @ 10 corner: 10 @ 50)!

labelCount
	^5!

maximumValue
	| total |
	total _ self model values
				inject: 0
				into: [ :sum :value | sum + value].
	^(total / self labelCount roundTo: self units) * self labelCount!

numberOfColumns
	^self model size!

positionFor: value
	^self barFrame height * value / self maximumValue!

units
	^100! !

!BarChartView methodsFor: 'displaying'!

displayBars
	| height bar corner |
	corner _ self barFrame bottomLeft.
	self model keys
			do: [ :key |
					height _ self positionFor: (self model at: key).
					bar _ corner-(0 @ height) extent:
							(self barFrame width/self numberOfColumns-10) @ height.
					Display black: bar.
					Display fill: (bar insetBy: 2@2)
							mask: Form darkGray.
					corner _ corner + ((self barFrame width/self numberOfColumns) @ 0)]!

displayView
	self displayXLabels.
	self displayYLabels.
	self displayBars!

displayXLabels
	| corner |
	corner _ self barFrame bottomLeft.
	self model keys do:
		[ :key |
			key displayAt: corner
							+(self barFrame width-20/(self numberOfColumns*2)
														@20)
									-(key asDisplayText boundingBox extent/2).
			corner _ corner + ((self barFrame width/self numberOfColumns)@0)]!

displayYLabels
	| count label increment |
	count _ self labelCount.
	increment _ self maximumValue / count.
	label _ 0.
	count timesRepeat: 
		[label printString displayAt: (self barFrame bottomLeft - (35 @ ((self positionFor: label)+7))).
		 label _ label + increment]!

update: aParameter
	self display! !

View subclass: #FinancialHistoryView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Demo-FinancialTools'!


!FinancialHistoryView methodsFor: 'controller access'!

defaultControllerClass

	^FinancialHistoryController! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FinancialHistoryView class
	instanceVariableNames: ''!


!FinancialHistoryView class methodsFor: 'instance creation'!

open: aFHModel
	| aFHView aBCView topView |
	topView _ StandardSystemView new.
	topView model: aFHModel.
	topView borderWidth: 2.
	topView insideColor: Form lightGray.
	topView label: 'Financial History'.
	topView minimumSize: 400@300.
	
	aFHView _ self new model: aFHModel.
	topView addSubView: aFHView.

	aBCView _ BarChartView new model: aFHModel expenditures.
	aBCView window: (10@10 extent: 35@35).
	aBCView insideColor: Form white.
	aBCView borderWidth: 2.
	aBCView controller: NoController new.
	aFHView addSubView: aBCView.

	aBCView _ BarChartView new model: aFHModel incomes.
	aBCView window: (55@10 extent: 35@35).
	aBCView insideColor: Form white.
	aBCView borderWidth: 2.
	aBCView controller: NoController new.
	aFHView addSubView: aBCView.

	topView controller open! !