'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 24 July 1995 at 3:40:05 pm'!

((CxxSystemOrganization tree childNamed: 'parray') ~= nil) ifTrue: [
	(CxxSystemOrganization tree childNamed: 'parray') destroyFiles]!

Heaper subclass: #PrimArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!
PrimArray comment:
'Array objects in smalltalk vary in size, while in x++ they allocate separate storage for the varying part.  Thus they require very different Recipes.

Recipes are normally registered by the class'' associated Recipe class at its own initialization time.  In smalltalk, the Array classes share a common Recipe class, (STArrayRecipe), which registers an instance of itself for each appropriate concrete subclass of PrimArray.X.

UInt1Arrays are odd in smalltalk, and get normal recipes generated by the normal smalltalk stubble mechanism

(WeakPtrArrays are not in the browser because they currently have no clients and may disappear.)'!

(PrimArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #DEFERRED; add: #EQ; yourself)!

!PrimArray methodsFor: 'accessing'!

{void CLIENT login} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."
	self subclassResponsibility!

{Int32 CLIENT login} count
	"How many elements the array can hold"
	
	^self basicSize!

{Heaper | NULL} fetchValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."
	self subclassResponsibility!

{Heaper CLIENT login} getValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."
	
	
	| result {Heaper} |
	result _ self fetchValue: index.
	result == NULL ifTrue: [Heaper BLAST: #NotInTable].
	^result!

{PrimSpec CLIENT} spec
	"A description of the kinds of things which can be stored in this array"
	
	self subclassResponsibility!

{Heaper CLIENT} valueAt: index {Int32}
	"Get an individual value from the array. It may be a primitive data type, or a pointer to an object. Blasts if it is NULL."
	
	| result {Heaper} |
	Ravi knownBug. "primitive types stuff"
	result := self fetchValue: index.
	result == NULL ifTrue:
		[Heaper BLAST: #MustNotBeNull].
	^result! !

!PrimArray methodsFor: 'destruction'!

{void} destroy! !

!PrimArray methodsFor: 'testing'!

{BooleanVar} contentsEqual: other {PrimArray}
	"Whether the two ranges contain semantically the same values. Two non-NULL pointers match iff the Heapers they point to are isEqual. Two integers match iff they have the same value, even though they may be represented as different sizes. Two floats likewise."
	
	self subclassResponsibility!

{UInt32} contentsHash
	"A hash of the entire contents of the array. If two arrays are contentsEqual, they will have the same contentsHash."

	^self elementsHash! !

!PrimArray methodsFor: 'bulk accessing'!

{void CLIENT login} at: to {Int32}
	storeMany: source {PrimArray}
	with: count {Int32 default: -1}
	with: from {Int32 default: Int32Zero}
	"Copy n elements from the source array into this one. The source array must be of a compatible type."

	|  delta {Int32} stop |
	delta _ to - from.
	count == -1 ifTrue: [stop _ source count] ifFalse: [stop _ from + count].
	(stop > source count or: [stop + delta > self count]) 
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	(self == source and: [from < to])
		ifTrue: [stop to: from+1 by: -1 do: [ :i | self basicAt: i + delta put: (source basicAt: i)]]
		ifFalse: [from+1 to: stop do: [ :i | self basicAt: i + delta put: (source basicAt: i)]]!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	with: before {Int32 default: Int32Zero}
	with: after {Int32 default: Int32Zero}
	"Make a copy of a piece of this array.  'Count' defaults to the count of the array minus 'start'.  'start' defaults to the beginning of the array, so with arguments, copy just copies the entire array.  'Before' and 'after' both default to 0.  They describe the count of nulls/0s to be added in the result array before the copied elements or after the copied elements.  For example, to copy everything after 10 into a new array leaving room for 20 elements, the incantation would be:
		array->copy(-1,10,20)"
	
	| copyCount {Int32} |
	count < Int32Zero
		ifTrue: [copyCount _ self count - start]
		ifFalse:
			[copyCount _ count.
			start + copyCount > self count
				ifTrue: [Heaper BLAST: #IndexOutOfBounds]].
	^self class create: copyCount + before + after with: self with: start with: copyCount with: before!

{void CLIENT} copyFromBuffer: buffer {void star}
	with: start {Int32 default: Int32Zero}
	with: count {Int32 default: -1}

	[self knownBug. "doesn't check size and does the wrong thing in some cases for defautl count"
	self at: start storeElements: buffer with: Int32Zero with: count] smalltalkOnly.
	[Eric shouldImplement] translateOnly!

{PrimArray} copyGrow: after {Int32}
	"Make a copy of the array into a larger array.  The array has 'after' slots after the copied elements."
	
	^self copy: self count with: Int32Zero with: Int32Zero with: after!

{void} copyToBuffer: buffer {void star} with: size {Int32} with: count {Int32 default: -1} with: start {Int32 default: Int32Zero}

	[self knownBug. "doesn't check size"
	buffer at: Int32Zero storeMany: self with: count with: start] smalltalkOnly.
	[Eric shouldImplement] translateOnly!

{Int32} indexOf: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}
	"The index of the nth occurrence of the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self subclassResponsibility!

{Int32} indexOfElements: other {PrimArray}
	with: valueCount {Int32 default: -1}
	with: valueStart {Int32 default: Int32Zero}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of the given sequence of values at or after (before if n is negative) the given starting index, or -1 if there is none. Negative numbers for start are relative to the end of the array."
	
	| valueN result n forward |
	valueCount < 0
		ifTrue: [valueN := other basicSize]
		ifFalse: [valueCount > other basicSize
				ifTrue: [Heaper BLAST: #IndexOutOfBounds].
			valueN := valueCount].
	start >= 0
		ifTrue: [result := start]
		ifFalse: [result := self basicSize + start].
	n := nth abs.
	forward := nth > 0.
		[(forward ifTrue: [result > self basicSize - valueN] ifFalse: [result < 0]) ifTrue:
			[^-1].
		(self elementsEqual: result with: other with: valueStart with: valueN) ifTrue:
			[n := n - 1.
			n = 0 ifTrue:
				[^result]].
		forward
			ifTrue: [result := result + 1]
			ifFalse: [result := result - 1]] repeat!

{Int32} indexPast: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self unimplemented!

{void CLIENT login} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	"Set a range of elements to have the same value"

	self subclassResponsibility! !

!PrimArray methodsFor: 'bulk testing'!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	with: n {Int32 default: -1}
	"Whether the two ranges contain the same values, using the criteria defined in contentsEqual"
	
	self subclassResponsibility!

{UInt32} elementsHash: count {Int32 default: -1} with: start {Int32 default: Int32Zero} 
	"A hash of the range of values out of the array. If two ranges are elementsEqual, they will have the same hash. For data values, additional zeros on the end make no difference to the hash."
	
	self subclassResponsibility! !

!PrimArray methodsFor: 'smalltalk: defaults'!

{void CLIENT login} at: to {Int32}
	storeMany: other {PrimArray}
	"Copy n elements from the other array into this one. The other array must be of a compatible type."
	
	^self at: to storeMany: other with: -1 with: 0!

{void CLIENT login} at: to {Int32}
	storeMany: other {PrimArray}
	with: count {Int32 default: -1}
	"Copy n elements from the other array into this one. The other array must be of a compatible type."
	
	^self at: to storeMany: other with: count with: 0!

{PrimArray CLIENT login} copy
	
	^self copy: -1 with: 0 with: 0 with: 0!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	
	^self copy: count with: 0 with: 0 with: 0!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	
	^self copy: count with: start with: 0 with: 0!

{PrimArray CLIENT login} copy: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}
	with: before {Int32 default: Int32Zero}
	
	^self copy: count with: start with: before with: 0!

{void} copyToBuffer: buffer {void start} with: size {Int32}
	self copyToBuffer: buffer with: size with: -1 with: Int32Zero!

{void} copyToBuffer: buffer {void start} with: size {Int32} with: count {Int32 default: -1}
	self copyToBuffer: buffer with: size with: count with: 0!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	
	^self elementsEqual: here with: other with: 0 with: -1!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	
	^self elementsEqual: here with: other with: there with: -1!

{UInt32} elementsHash
	^self elementsHash: -1 with: 0!

{UInt32} elementsHash: count {Int32 default: -1}
	^self elementsHash: count with: 0!

{Int32} indexOf: value {Heaper}
	"The index of the nth occurrence of the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self indexOf: value with: 0 with: 1!

{Int32} indexOf: value {Heaper}
	with: start {Int32 default: Int32Zero}
	"The index of the nth occurrence of the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self indexOf: value with: start with: 1!

indexOfElements: other
	^self indexOfElements: other with: -1 with: 0 with: 0 with: 1!

indexOfElements: other with: valueCount
	^self indexOfElements: other with: valueCount with: 0 with: 0 with: 1!

indexOfElements: other with: valueCount with: valueStart 
	^self indexOfElements: other with: valueCount with: valueStart with: 0 with: 1!

indexOfElements: other with: valueCount with: valueStart with: start
	^self indexOfElements: other with: valueCount with: valueStart with: start with: 1!

{Int32} indexPast: value {Heaper}
	with: start {Int32 default: Int32Zero}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	^ self indexPast: value with: start with: 1!

{void CLIENT login} storeAll
	^self storeAll: NULL with: -1 with: 0!

{void CLIENT login} storeAll: value {Heaper default: NULL}
	^self storeAll: value with: -1 with: 0!

{void CLIENT login} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	^self storeAll: value with: count with: 0! !

!PrimArray methodsFor: 'printing'!

{void} printOn: oo {ostream reference}

	| before |
	before := '['.
	self count = 0 ifTrue:
		[oo << '[empty']
	ifFalse:
		[1 to: self basicSize do: [ :i {UInt32} |
			oo << before << (self basicAt: i).
			before := ' ']].
	oo << ']'! !

!PrimArray methodsFor: 'smalltalk: protected: implementation'!

{Int32} indexOfObject: value {IntegerVar | IEEE64 | Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	| result n |
	start < 0
		ifTrue: [result := self basicSize + start + 1.
			result < 0
				ifTrue: [Heaper BLAST: #IndexOutOfBounds]]
		ifFalse: [result := start + 1].
	result > self basicSize
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	nth >= 0 ifTrue:
		[n := nth.
			[value = (self basicAt: result) ifTrue:
				[n := n - 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result + 1.
		result < self basicSize] whileTrue.
		^-1]
	ifFalse:
		[n := nth.
			[value = (self basicAt: result) ifTrue:
				[n := n + 1.
				n == 0
					ifTrue: [^result]].
			result := result - 1.
		result > 0] whileTrue.
		^-1]!

{Int32} indexPastObject: value {IntegerVar | IEEE64 | Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	| result n |
	(self basicSize = 0 or: [nth = 0]) ifTrue: [^-1].
	start < 0
		ifTrue: [result := self basicSize + start + 1]
		ifFalse: [result := start + 1].
	(result <= 0 or: [result > self basicSize])
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	nth > 0 ifTrue:
		[n := nth.
			[value = (self basicAt: result) ifFalse:
				[n := n - 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result + 1.
		result <= self basicSize] whileTrue.
		^-1]
	ifFalse:
		[n := nth.
			[value = (self basicAt: result) ifFalse:
				[n := n + 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result - 1.
		result > 0] whileTrue.
		^-1]! !

!PrimArray methodsFor: 'create'!

create: count {Int32} with: buffer {void star}
	"The buffer had better just behave like a Smalltalk array"

	super create.
	(buffer class == self class or: [buffer isKindOf: ByteString]) assert: 'Mismatched buffer types'.
	self at: Int32Zero storeMany: buffer! !

!PrimArray methodsFor: 'smalltalk: c arrays'!

at: i
	"act like a C array"
	
	^self basicAt: i + 1!

at: i put: value
	"act like a C array"
	
	self basicAt: i + 1 put: value.
	^value!

basicAt: index 
	"Answer the value of an indexable field in the receiver. Fail if the 
	argument index is not an Integer or is out of bounds. Essential. See 
	documentation in Object metaclass.  Do not override this message in any 
	subclass."

	<primitive: 60>
	index isInteger ifTrue: [^Heaper BLAST: #NotInTable].
	index respondsToArithmetic
		ifTrue: [^self basicAt: index asSmallInteger]
		ifFalse: [^self nonIntegerIndexError: index]!

basicAt: index put: value 
	"Store the second argument value in the indexable field of the receiver 
	indicated by index. Fail if the index is not an Integer or is out of bounds. Or 
	fail if the value is not of the right type for this kind of collection. Answer 
	the value that was stored. Essential. See documentation in Object 
	metaclass. Do not override in a subclass."

	<primitive: 61>
	index isInteger
		ifTrue: [(index >= 1 and: [index <= self basicSize])
					ifTrue: [^self improperStoreError]
					ifFalse: [^Heaper BLAST: #NotInTable]].
	index respondsToArithmetic
		ifTrue: [^self basicAt: index asSmallInteger put: value]
		ifFalse: [^self nonIntegerIndexError: index]! !

!PrimArray methodsFor: 'smalltalk: gc'!

{void} markInstances: count {IntegerVar} 
	"Don't do anything since most subclasses are bits.  Subclasses reimplement this."! !

!PrimArray methodsFor: 'generated:'!

actualHashForEqual ^self asOop!

isEqual: other ^self == other! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PrimArray class
	instanceVariableNames: ''!

(PrimArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #DEFERRED; add: #EQ; yourself)!

!PrimArray class methodsFor: 'smalltalk: creation'!

create: len
	^ (self new: len) create: len!

create: len with: other
	"myAllocations _ myAllocations + 1."
	^ (self new: len) create: len with: other!

create: len with: other with: a with: b with: c
	"myAllocations _ myAllocations + 1."
	^ (self new: len) create: len with: other with: a with: b with: c! !PrimArray subclass: #PrimDataArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-parray'!
PrimDataArray comment:
'A common superclass for primitive arrays of basic data types (i.e. bits of some kind)'!

(PrimDataArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #DEFERRED; yourself)!

!PrimDataArray methodsFor: 'bulk testing'!

{Int32} compare: other {PrimDataArray}
	with: count {Int32 default: -1}
	with: here {UInt32 default: Int32Zero}
	with: there {Int32 default: Int32Zero}
	"Return -1, 0, or +1 according to whether the elements in the specified span of this array are lexically less than, equal to, or greater than the specified span of the other. The other array must be of a compatible type. If the count is negative or goes beyond the end of either array, then the shorter array is considered to be extended with zeros.
	NOTE: Because of zero extension, this is not the same as elementsEqual; it is possible that a->compare (b) == 0 even though !! a->contentsEqual (b)"
	
	| n {Int32} longer index |
	n := (other basicSize - there) min: (self basicSize - here).
	(count >= 0 and: [count < n])
		ifTrue: [n := count].
	1 to: n do: [ :i | | a b |
		(a := self basicAt: here + i) < (b := other basicAt: there + i)
			ifTrue: [^-1].
		a > b ifTrue: [^1]].
	"past the end. check if a count was specified or if they are the same length"
	(count = n or: [(other basicSize - there) = (self basicSize - here)])
		ifTrue: [^0].
	"figure out which is longer and look for a nonzero element"
	n = (other basicSize - there) ifTrue:
		[longer := self.
		index := here + n + 1]
	ifFalse:
		[longer := other.
		index := there + n + 1].
	count >= Int32Zero
		ifTrue: [n := (count - n + index) min: longer basicSize]
		ifFalse: [n := longer basicSize].
	[index <= n] whileTrue:
		[ | val |
		(val := longer basicAt: index) < 0 ifTrue:
			[^longer == self ifTrue: [-1] ifFalse: [1]].
		val > 0 ifTrue:
			[^longer == self ifTrue: [1] ifFalse: [-1]].
		index := index + 1].
	^0!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	with: count {Int32 default: -1}
	
	| n {Int32} |
	n := (other basicSize - there) min: (self basicSize - here).
	count > n ifTrue:
		[Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	1 to: n do: [ :i |
		(self basicAt: here + i) = (other basicAt: there + i) ifFalse:
			[^false]].
	^true!

{UInt32} elementsHash: count {Int32 default: -1} with: start {Int32 default: Int32Zero}
	"A hash of the range of values out of the array"
	
	self subclassResponsibility! !

!PrimDataArray methodsFor: 'smalltalk: defaults'!

{void} at: to {Int32}
	addElements: other {PrimDataArray}
	
	^self at: to addElements: other with: -1 with: 0!

{void} at: to {Int32}
	addElements: other {PrimDataArray}
	with: count {Int32 default: -1}
	
	^self at: to addElements: other with: count with: 0!

{void} at: to {Int32}
	subtractElements: other {PrimDataArray}
	
	^self at: to subtractElements: other with: -1 with: 0!

{void} at: to {Int32}
	subtractElements: other {PrimDataArray}
	with: count {Int32 default: -1}
	
	^self at: to subtractElements: other with: count with: 0!

{Int32} compare: other {PrimArray}

	^self compare: other with: -1 with: 0 with: 0!

{Int32} compare: other {PrimArray}
	with: count {Int32 default: -1}
	
	^self compare: other with: count with: 0 with: 0!

{Int32} compare: other {PrimArray}
	with: count {Int32 default: -1}
	with: here {UInt32 default: Int32Zero}
	
	^self compare: other with: count with: here with: 0! !

!PrimDataArray methodsFor: 'testing'!

{BooleanVar} contentsEqual: other {PrimArray}
	
	self basicSize = other basicSize ifFalse: [^false].
	1 to: self basicSize do: [ :i |
		(self basicAt: i) = (other basicAt: i)
			ifFalse: [^false]].
	^true! !

!PrimDataArray methodsFor: 'bulk accessing'!

{void} at: to {Int32}
	addElements: other {PrimDataArray}
	with: count {Int32 default: -1}
	with: from {Int32 default: Int32Zero}
	
	| n {Int32} |
	n := (other basicSize - from) min: (self basicSize - to).
	count > n ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	count >= 0 ifTrue: [n := count].
	1 to: n do: [ :i | self basicAt: to + i
		put: (other basicAt: from + i) + (self basicAt: to + i)]!

{void} at: to {Int32}
	subtractElements: other {PrimDataArray}
	with: count {Int32 default: -1}
	with: from {Int32 default: Int32Zero}
	
	| n {Int32} |
	n := (other basicSize - from) min: (self basicSize - to).
	count > n ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	count >= 0 ifTrue: [n := count].
	1 to: n do: [ :i | self basicAt: to + i
		put: (self basicAt: to + i) - (other basicAt: from + i)]! !PrimDataArray subclass: #PrimFloatArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!

(PrimFloatArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #CONCRETE; yourself)!

!PrimFloatArray methodsFor: 'accessing'!

{void} at: index {Int32} storeFloat: value {IEEE64}
	"Store a floating point value"
	
	self basicAt: 1 + index put: value!

{Int32 CLIENT} bitCount
	"Return the maximum precision that can be stored in this array."
	
	self subclassResponsibility!

{IEEE64} floatAt: index {Int32}
	"Get an actual floating point number"
	
	^self basicAt: 1 + index! !

!PrimFloatArray methodsFor: 'testing'!

{UInt32} elementsHash: count {Int32 default: -1} with: start {Int32 default: Int32Zero}
	
	^self getCategory hashForEqual bitXor: (FHash fastHash.UInt32: count)! !

!PrimFloatArray methodsFor: 'bulk accessing'!

{Int32} indexOf: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}

	^self indexOfObject: (value cast: PrimFloatValue) asIEEE64 with: start with: n!

{Int32} indexPast: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}

	^self indexPastObject: (value cast: PrimFloatValue) asIEEE64 with: start with: n! !

!PrimFloatArray methodsFor: 'smalltalk: passe'!

{Int32} precision

	self passe "bitCount"! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PrimFloatArray class
	instanceVariableNames: ''!

(PrimFloatArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #CONCRETE; yourself)!

!PrimFloatArray class methodsFor: 'creation'!

{PrimFloatArray CLIENT} zeros: precision {Int32} with: count {Int32}
	"Make an array initialized to zero values"

	precision == 64 ifTrue: [^IEEE64Array make: count].
	precision == 32 ifTrue: [^IEEE32Array make: count].
	Heaper BLAST: #UnimplementedPrecision.
	"compiler fodder"
	^NULL! !PrimFloatArray variableSubclass: #IEEE32Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-parray'!

(IEEE32Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #COPY; yourself)!

!IEEE32Array methodsFor: 'accessing'!

{void} at: index {Int32} storeIEEE32: value {IEEE32}
	"Store a floating point value"
	
	self basicAt: 1 + index put: value!

{void} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."

	self basicAt: 1 + index put: (value cast: PrimIEEE32) asIEEE32!

{Int32 CLIENT} bitCount
	
	^32!

{Heaper | NULL} fetchValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."

	^PrimIEEE32 make: (self basicAt: 1 + index)!

{IEEE32} iEEE32At: index {Int32}
	"Get an actual floating point number"
	
	^self basicAt: 1 + index!

{PrimSpec} spec

	^PrimSpec iEEE32! !

!IEEE32Array methodsFor: 'create'!

create: count {Int32}

	super create.
	1 to: count do: [ :i |
		self basicAt: i put: 0.0]!

create: size {Int32}
	with: from {PrimFloatArray}
	with: start {Int32}
	with: count {Int32}
	with: offset {Int32}

	super create.
	1 to: offset do: [ :i |
		self basicAt: i put: 0.0].
	1 + offset to: count + offset do: [ :i |
		self basicAt: i put: (from basicAt: i + start)].
	1 + size + offset to: size do: [ :i |
		self basicAt: i put: 0.0].! !

!IEEE32Array methodsFor: 'bulk accessing'!

{void} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}

	| f {IEEE64Var} n {Int32} |
	n := self basicSize - start.
	count > n
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	value == NULL
		ifTrue: [f := 0.0]
		ifFalse: [f := (value cast: PrimFloatValue) asIEEE32].
	1 to: n do: [ :i |
		self basicAt: start + i put: f]! !

!IEEE32Array methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.! !PrimFloatArray variableSubclass: #IEEE64Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-parray'!

(IEEE64Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #COPY; yourself)!

!IEEE64Array methodsFor: 'accessing'!

{void} at: index {Int32} storeIEEE64: value {IEEE64}
	"Store a floating point value"
	
	self basicAt: 1 + index put: value!

{void} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."

	self basicAt: 1 + index put: (value cast: PrimIEEE64) asIEEE64!

{Int32 CLIENT} bitCount
	
	^64!

{Heaper | NULL} fetchValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."

	^PrimIEEE64 make: (self basicAt: 1 + index)!

{IEEE64} iEEE64At: index {Int32}
	"Get an actual floating point number"
	
	^self basicAt: 1 + index!

{PrimSpec} spec

	^PrimSpec iEEE64! !

!IEEE64Array methodsFor: 'create'!

create: count {Int32}

	super create.
	1 to: count do: [ :i |
		self basicAt: i put: 0.0d]!

create: size {Int32}
	with: from {PrimFloatArray}
	with: start {Int32}
	with: count {Int32}
	with: offset {Int32}

	super create.
	1 to: offset do: [ :i |
		self basicAt: i put: 0.0d].
	1 + offset to: count + offset do: [ :i |
		self basicAt: i put: (from basicAt: i + start)].
	1 + size + offset to: size do: [ :i |
		self basicAt: i put: 0.0d].! !

!IEEE64Array methodsFor: 'bulk accessing'!

{void} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}

	| f {IEEE64Var} n {Int32} |
	n := self basicSize - start.
	count > n
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	value == NULL
		ifTrue: [f := 0.0d]
		ifFalse: [f := (value cast: PrimFloatValue) asIEEE64].
	1 to: n do: [ :i |
		self basicAt: start + i put: f]! !

!IEEE64Array methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.! !PrimDataArray subclass: #PrimIntegerArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!
PrimIntegerArray comment:
'A common X++ superclass for primitive arrays of integer types.  This provides integer accessing polymorphic between integer representations.  This distinctions does not make any sense across the wire.'!

(PrimIntegerArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #DEFERRED; yourself)!

!PrimIntegerArray methodsFor: 'accessing'!

{PrimIntegerArray} at: index {Int32}
	hold: value {IntegerVar}
	with: canModify {BooleanVar default: false}
	"Store a new value into the array at the given index. If the value does not fit into the range that can be stored here, return an array of a kind that will accept it. If the index is past the end, return a larger array.
	If canModify, then returns a newly created array only if the value will not fit into this one, otherwise will always return a new array."
	
	| result {PrimIntegerArray} |
	index < Int32Zero ifTrue:
		[Heaper BLAST: #IndexOutOfBounds].
	index >= self count ifTrue:
		[result := ((self spec canHold: value)
				ifTrue: [self spec] ifFalse: [PrimSpec toHold: value])
			copyGrow: self with: index + 1 - self count]
	ifFalse: [(self spec canHold: value) ifTrue:
		[canModify
			ifTrue: [result := self]
			ifFalse: [result := self copy]]
	ifFalse:
		[result := (PrimSpec toHold: value) copy: self]].
	result at: index storeInteger: value.
	^result!

{void} at: index {Int32} storeInteger: value {IntegerVar}
	"Store an integer value"
	
	self basicAt: 1 + index put: value!

{Heaper | NULL} fetchValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."

	^PrimIntValue make: (self basicAt: 1 + index)!

{IntegerVar} integerAt: index {Int32}
	"Get an actual integer value"
	
	^self basicAt: 1 + index! !

!PrimIntegerArray methodsFor: 'testing'!

{UInt32} elementsHash: count {Int32 default: -1} with: start {Int32 default: Int32Zero}


	| n {Int32} |
	n := self basicSize - start.
	count > n ifTrue:
		[Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	n == Int32Zero ifTrue:
		[^FHash fastHash.UInt32: 17]
	ifFalse: [n == 1 ifTrue:
		[^FHash fastHash.UInt32: (self basicAt: 1)]
	ifFalse:
		[^((FHash fastHash.UInt32: n)
			bitXor: (FHash fastHash.UInt32: ((self basicAt: 1 + start) bitAnd: 65535)))
			bitXor: (FHash fastHash.UInt32: ((self basicAt: start + n) bitAnd: 65535))]]! !

!PrimIntegerArray methodsFor: 'smalltalk: defaults'!

{PrimIntegerArray} at: index {Int32}
	hold: value {IntegerVar}
	
	^self at: index hold: value with: false!

{Int32} indexOfInteger: value {IntegerVar}
	^self indexOfInteger: value with: 0 with: 1!

{Int32} indexOfInteger: value {IntegerVar}
	with: start {Int32 default: Int32Zero}
	^self indexOfInteger: value with: start with: 1!

{Int32} indexPastInteger: value {IntegerVar}
	^self indexPastInteger: value with: 0 with: 1!

{Int32} indexPastInteger: value {IntegerVar}
	with: start {Int32 default: Int32Zero}
	^self indexPastInteger: value with: start with: 1! !

!PrimIntegerArray methodsFor: 'bulk accessing'!

{Int32} indexOf: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}

	^self indexOfObject: (value cast: PrimIntValue) asIntegerVar with: start with: n!

{Int32} indexOfInteger: value {IntegerVar}
	with: start {Int32 default: Int32Zero}
	with: count {Int32 default: 1}

	^self indexOfObject: value with: start with: count!

{Int32} indexPast: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}

	^self indexPastObject: (value cast: PrimIntValue) asIntegerVar with: start with: n!

{Int32} indexPastInteger: value {IntegerVar}
	with: start {Int32 default: Int32Zero}
	with: count {Int32 default: 1}

	^self indexPastObject: value with: start with: count!

{void} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}

	| k {IntegerVar} n {Int32} |
	n := self basicSize - start.
	count > n
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	value == NULL
		ifTrue: [k := 0]
		ifFalse: [k := (value cast: PrimIntValue) asIntegerVar].
	1 to: n do: [ :i |
		self basicAt: start + i put: k]! !

!PrimIntegerArray methodsFor: 'create'!

create: count {Int32}

	super create.
	1 to: count do: [ :i | self basicAt: i put: 0]!

create: size {Int32}
	with: from {PrimIntegerArray}
	with: start {Int32}
	with: count {Int32}
	with: offset {Int32}

	super create.
	1 to: offset do: [ :i |
		self basicAt: i put: 0].
	self at: offset storeMany: from with: count with: start.
	1 + count + offset to: size do: [ :i |
		self basicAt: i put: 0].! !PrimIntegerArray variableSubclass: #IntegerVarArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!

(IntegerVarArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #(COPY xpp ); add: #ON.CLIENT; add: #MANUAL.RECIPE; add: #CONCRETE; yourself)!

!IntegerVarArray methodsFor: 'accessing'!

{void} at: index {Int32} storeIntegerVar: value {IntegerVar}
	"Store an integer value"
	
	self basicAt: 1 + index put: value!

{void} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."

	self basicAt: 1 + index put: (value cast: PrimIntValue) asIntegerVar!

{IntegerVar} integerVarAt: index {Int32}
	"Get an actual integer value"
	
	^self basicAt: 1 + index!

{PrimSpec} spec
	
	^PrimSpec integerVar! !

!IntegerVarArray methodsFor: 'hooks:'!

{void RECEIVE.HOOK} receiveIVArray: rcvr {Rcvr}
	0 to: self count - 1 do: [ :i {Int32} | self at: i storeIntegerVar: rcvr receiveIntegerVar]!

{void SEND.HOOK} sendIVArray: xmtr {Xmtr}
	xmtr sendInt32: self count.
	1 to: self count do: [ :i {Integer} |
		xmtr sendIntegerVar: (self basicAt: i)].! !

!IntegerVarArray methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	self receiveIVArray: receiver.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	self sendIVArray: xmtr.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IntegerVarArray class
	instanceVariableNames: ''!

(IntegerVarArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #(COPY xpp ); add: #ON.CLIENT; add: #MANUAL.RECIPE; add: #CONCRETE; yourself)!

!IntegerVarArray class methodsFor: 'creation'!

{IntegerVarArray CLIENT} zeros: count {Int32}

	^self create: count! !

!IntegerVarArray class methodsFor: 'smalltalk: self copy'!

create.Rcvr: rcvr {Rcvr}
	| count result |
	count _ rcvr receiveIntegerVar.
	result _ self create: count.
	0 to: count - 1 do: [ :i |
		result at: i put: (rcvr receiveIntegerVar)].
	^result! !

!IntegerVarArray class methodsFor: 'smalltalk: smalltalk initialization'!

staticTimeNonInherited
	STArrayRecipe create: self with: (Smalltalk associationAt: #XppCuisine)! !

!IntegerVarArray class methodsFor: 'promise stuff'!

exportName
	^'HumberArray'! !

!IntegerVarArray class methodsFor: 'smalltlak: passe'!

make: count {Int32}

	self passe. "use zeros:"!

make: size {UInt32} with: old {IntegerVarArray}
	self passe.
	^self create: size with: old! !PrimIntegerArray subclass: #PrimIntArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!
PrimIntArray comment:
'A common superclass for primitive arrays of integral types'!

(PrimIntArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #DEFERRED; yourself)!

!PrimIntArray methodsFor: 'accessing'!

{Int32 CLIENT} bitCount
	"Return the maximum precision that can be stored in this array.  The number will be negative for Signed arrays."
	
	self subclassResponsibility! !

!PrimIntArray methodsFor: 'smalltalk: passe'!

{Int32} precision

	self passe "bitCount"! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PrimIntArray class
	instanceVariableNames: ''!

(PrimIntArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #ON.CLIENT; add: #DEFERRED; yourself)!

!PrimIntArray class methodsFor: 'creation'!

{PrimIntArray CLIENT} zeros: precision {Int32} with: count {Int32}
	"Make an array initialized to zero values.  The values are signed if precision is negative."

	precision == 8 ifTrue: [^UInt8Array make: count].
	precision == 32 ifTrue: [^UInt32Array make: count].
	precision == -32 ifTrue: [^Int32Array make: count].
	Heaper BLAST: #UnimplementedPrecision.
	"compiler fodder"
	^NULL! !PrimIntArray variableSubclass: #Int32Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!

(Int32Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #MANUAL.RECIPE; add: #(COPY xpp ); yourself)!

!Int32Array methodsFor: 'accessing'!

{void} at: index {Int32} storeInt: value {Int32}
	"Store a 32 bit signed integer value"
	
	self basicAt: 1 + index put: value!

{void} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."

	self basicAt: 1 + index put: (value cast: PrimIntValue) asInt32!

{Int32 CLIENT} bitCount
	"Return the maximum precision that can be stored in this array.  The number will be negative for Signed arrays."
	
	^-32!

{Int32} intAt: index {Int32}
	"Get a 32 bit signed actual integer value"
	
	^self basicAt: 1 + index!

{PrimSpec} spec
	
	^PrimSpec int32! !

!Int32Array methodsFor: 'hooks:'!

{void RECEIVE.HOOK} receiveInt32Array: rcvr {Rcvr}

	0 almostTo: self count do: [ :i {Int32} |
		self at: i storeInt: rcvr receiveInt32]!

{void SEND.HOOK} sendInt32Array: xmtr {Xmtr}

	xmtr sendInt32: self count.
	0 almostTo: self count do: [ :i {Int32} |
		xmtr sendInt32: (self intAt: i)]! !

!Int32Array methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	self receiveInt32Array: receiver.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	self sendInt32Array: xmtr.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Int32Array class
	instanceVariableNames: ''!

(Int32Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #MANUAL.RECIPE; add: #(COPY xpp ); yourself)!

!Int32Array class methodsFor: 'smalltalk: initialization'!

staticTimeNonInherited

	STArrayRecipe create: self
		with: (Smalltalk associationAt: #XppCuisine)! !

!Int32Array class methodsFor: 'pseudo constructors'!

make: count {Int32}

	^self create: count! !PrimIntArray variableSubclass: #UInt32Array
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!

(UInt32Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #MANUAL.RECIPE; add: #(COPY xpp ); yourself)!

!UInt32Array methodsFor: 'accessing'!

{void} at: index {Int32} storeUInt: value {UInt32}
	"Store a 32 bit unsigned integer value"
	
	self basicAt: 1 + index put: value!

{void} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."

	self basicAt: 1 + index put: (value cast: PrimIntValue) asUInt32!

{Int32 CLIENT} bitCount
	"Return the maximum precision that can be stored in this array.  The number will be negative for Signed arrays."
	
	^32!

{PrimSpec} spec
	
	^PrimSpec uInt32!

{UInt32} uIntAt: index {Int32}
	"Get a 32 bit unsigned actual integer value"
	
	^self basicAt: 1 + index! !

!UInt32Array methodsFor: 'hooks:'!

{void RECEIVE.HOOK} receiveInt32Array: rcvr {Rcvr}

	0 to: self count - 1 do: [ :i {Int32} |
		self at: i storeInt: rcvr receiveUInt32]!

{void SEND.HOOK} sendInt32Array: xmtr {Xmtr}

	xmtr sendInt32: self count.
	1 to: self count do: [ :i {Int32} |
		xmtr sendUInt32: (self intAt: i)]! !

!UInt32Array methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	self receiveInt32Array: receiver.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	self sendInt32Array: xmtr.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

UInt32Array class
	instanceVariableNames: ''!

(UInt32Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #MANUAL.RECIPE; add: #(COPY xpp ); yourself)!

!UInt32Array class methodsFor: 'smalltalk: initialization'!

staticTimeNonInherited
	STArrayRecipe create: self with: (Smalltalk associationAt: #XppCuisine)! !

!UInt32Array class methodsFor: 'pseudo constructors'!

make: count {Int32}

	^self create: count! !PrimIntArray variableByteSubclass: #UInt8Array
	instanceVariableNames: ''
	classVariableNames: 'ProtoArray {UNKNOWN} '
	poolDictionaries: ''
	category: 'X++ PrimArrays'!

(UInt8Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #MANUAL.RECIPE; add: #(COPY xpp ); yourself)!

!UInt8Array methodsFor: 'accessing'!

{void} at: to {Int32}
	storeElements: other {PrimArray}
	with: from {Int32 default: Int32Zero}
	with: count {Int32 default: -1}
	"Copy count elements from the other array into this one at . The other array must be of a compatible type."

	| n |
	n _ count.
	n < 0 ifTrue: [n _ self count - to min: other count - from].
	self primReplaceFrom: to+1 to: to + n with: other startingAt: from+1 
	"This destructively replaces elements from start to stop in the receiver
	starting at index, repStart, in the collection, replacement.  Answer the
	receiver.  No range checks are performed - this may be primitively implemented."!

{void CLIENT login} at: to {Int32}
	storeMany: other {PrimArray}
	with: count {Int32 default: -1}
	with: from {Int32 default: Int32Zero}
	"Copy count elements from the other array into this one at . The other array must be of a compatible type."

	self primReplaceFrom: to+1 to: to + (count < 0 ifTrue: [other count] ifFalse: [count]) with: other startingAt: from+1 
	"This destructively replaces elements from start to stop in the receiver
	starting at index, repStart, in the collection, replacement.  Answer the
	receiver.  No range checks are performed - this may be primitively implemented."!

{void} at: index {Int32} storeUInt: value {UInt32}
	"Store a 32 bit unsigned integer value"
	
	self basicAt: 1 + index put: value!

{void} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."

	self basicAt: 1 + index put: (value cast: PrimIntValue) asUInt32!

{Int32 CLIENT} bitCount
	"Return the maximum precision that can be stored in this array.  The number will be negative for Signed arrays."
	
	^8!

{Character star} gutsOf
	"A pointer to the actual string.  Since other PrimArray allocation can happen while such a pointer is outstanding, and this can cause relocation (in C++), one must call noMoreGuts when you will no longer be using the pointer."
	
	^self!

{void} noMoreGuts
	"call this once for each call to gutsOf, in order to indicate that the pointer returned by gutsOf will no longer be used"!

{PrimSpec} spec
	
	^PrimSpec uInt8!

{UInt32} uIntAt: index {Int32}
	"Get a 32 bit unsigned actual integer value"
	
	^self basicAt: 1 + index! !

!UInt8Array methodsFor: 'printing'!

{void} printOn: oo {ostream reference}

	1 to: self basicSize do: [ :i |
		oo nextPut: (Character value: (self basicAt: i))]! !

!UInt8Array methodsFor: 'private: accessing'!

primReplaceFrom: start to: stop with: replacement startingAt: repStart 
	"This destructively replaces elements from start to stop in the receiver
	starting at index, repStart, in the collection, replacement.  Answer the
	receiver.  No range checks are performed - this may be primitively implemented."
	<primitive: 105>
	 start = (stop + 1) ifTrue: [^self].
	 self error: 'storeElements failed.'! !

!UInt8Array methodsFor: 'smalltalk: unneeded'!

asString
	| result |
	result _ String new: self count.
	result replaceFrom: 1 to: result size withIntegerArray: self startingAt: 1.
	^result
	
	"(UInt8Array string: 'foo') asString"!

{Int32} compareString: string {Character star} with: start {Int32 default: Int32Zero}
	"Return -1, 0, or 1 depending on whether the contents at the given index are less than, equal to, or greater than the string. If they match up to the end, then the shorter one is the lesser."
	
	| diff |
	1 to: (string size min: self basicSize - start) do: [ :i |
		(diff := (self basicAt: i + start) - (string basicAt: i)) < 0
			ifTrue: [^-1].
		diff > 0
			ifTrue: [^1]].
	(diff := (self basicSize - start) - string size) < 0
		ifTrue: [^-1].
	diff > 0
		ifTrue: [^1].
	^0! !

!UInt8Array methodsFor: 'create'!

create: count {Int32}

	super create!

create: size {Int32}
	with: from {PrimIntegerArray}
	with: start {Int32}
	with: count {Int32}
	with: offset {Int32}

	"Everything else starts a Zero."
	self at: offset storeMany: from with: count with: start.! !

!UInt8Array methodsFor: 'hooks:'!

{void RECEIVE.HOOK} receiveUInt8Array: rcvr {Rcvr}
	rcvr receiveData: self!

{void SEND.HOOK} sendUInt8Array: xmtr {Xmtr}
	xmtr sendInt32: self count.
	xmtr sendUInt8Data: self! !

!UInt8Array methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	self receiveUInt8Array: receiver.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	self sendUInt8Array: xmtr.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

UInt8Array class
	instanceVariableNames: ''!

(UInt8Array getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #MANUAL.RECIPE; add: #(COPY xpp ); yourself)!

!UInt8Array class methodsFor: 'smalltalk: init'!

staticTimeNonInherited

	STArrayRecipe create: self with: (Smalltalk associationAt: #XppCuisine).
	ProtoArray := self new! !

!UInt8Array class methodsFor: 'pseudo constructors'!

make: count {Int32}

	^self create: count!

{UInt8Array} string: string {Character star}

	^string shallowCopy asString changeClassToThatOf: ProtoArray! !

!UInt8Array class methodsFor: 'exceptions:'!

bomb.ReleaseGuts: CHARGE {UInt8Array}

	^[CHARGE noMoreGuts]! !PrimArray variableSubclass: #PtrArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'X++ PrimArrays'!

(PtrArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #(COPY xpp ); add: #ON.CLIENT; add: #MANUAL.RECIPE; add: #CONCRETE; yourself)!

!PtrArray methodsFor: 'smalltalk: defaults'!

{BooleanVar} elementsEQ: here {Int32}
	with: other {PrimArray}
	
	^self elementsEQ: here with: other with: 0 with: -1!

{BooleanVar} elementsEQ: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	
	^self elementsEQ: here with: other with: there with: -1!

{Int32} indexOfEQ: value {Heaper | NULL}
	"The index of the first occurrence of anything but the given value, or -1 if there is none."

	self indexOfEQ: value with: 0 with: 1!

{Int32} indexOfEQ: value {Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	"The index of the first occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	self indexOfEQ: value with: start with: 1! !

!PtrArray methodsFor: 'bulk testing'!

{BooleanVar} elementsEQ: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	with: count {Int32 default: -1}
	
	| n {Int32} |
	n := (other basicSize - there) min: (self basicSize - here).
	count > n
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	1 to: n do: [ :i |
		(self basicAt: here + i) == (other basicAt: there + i) ifFalse:
				[^false]].
	^true!

{BooleanVar} elementsEqual: here {Int32}
	with: other {PrimArray}
	with: there {Int32 default: Int32Zero}
	with: count {Int32 default: -1}
	
	| n {Int32} |
	n := (other basicSize - there) min: (self basicSize - here).
	count > n ifTrue:
		[Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	1 to: n do: [ :i | | a b |
		((a := self basicAt: here + i) == (b := other basicAt: there + i)
			or: [a ~~ NULL and: [b ~~ NULL and: [a isEqual: b]]]) ifFalse:
				[^false]].
	^true!

{UInt32} elementsHash: count {Int32 default: -1} with: start {Int32 default: Int32Zero}

	| result n |
	result := 0.
	n := (count == -1) ifTrue: [self basicSize - start] ifFalse: [count].
	1 to: n do: [ :i |
		result := result bitXor: (self basicAt: i + start) hashForEqual].
	^ result.! !

!PtrArray methodsFor: 'testing'!

{BooleanVar} contentsEQ: other {PtrArray}

	self basicSize ~~ other basicSize ifTrue: [^ false].
	1 to: self basicSize do: [ :i {UInt32} |
		(self basicAt: i) == (other basicAt: i)
			ifFalse: [^false]].
	^true!

{BooleanVar} contentsEqual: other {PtrArray}

	self basicSize ~~ other basicSize ifTrue: [^ false].
	1 to: self basicSize do: [ :i {UInt32} | | a b |
		((a := self basicAt: i) == (b := other basicAt: i)
				or: [a ~~ NULL and: [b ~~ NULL and: [a isEqual: b]]])
			ifFalse: [^false]].
	^true!

{UInt32} contentsHash
	| result {UInt32} |
	result _ self count * 43.
	1 to: self count do: [ :i | | p |
		(p := self basicAt: i) ~~ NULL
			ifTrue: [result _ result bitXor: p hashForEqual]].
	^result! !

!PtrArray methodsFor: 'creation'!

create: count {Int32}

	super create.
	1 to: count do: [ :i | self basicAt: i put: NULL]!

create: size {Int32}
	with: from {PrimIntegerArray}
	with: start {Int32}
	with: count {Int32}
	with: offset {Int32}

	super create.
	1 to: offset do: [ :i |
		self basicAt: i put: NULL].
	self at: offset storeMany: from with: count with: start.
	1 + count + offset to: size do: [ :i |
		self basicAt: i put: NULL].! !

!PtrArray methodsFor: 'hooks:'!

{void RECEIVE.HOOK} receiveIPtrArray: rcvr {Rcvr}
	0 to: self count - 1 do: [ :i {Int32} | self at: i store: rcvr receiveHeaper]!

{void SEND.HOOK} sendPtrArray: xmtr {Xmtr}
	xmtr sendInt32: self count.
	1 to: self count do: [ :i {Int32} |
		xmtr sendHeaper: (self basicAt: i)].! !

!PtrArray methodsFor: 'smalltalk: destruct'!

delete!

{void} destruct
	"Classes should implement this message rather than a destructor. We use this so 
	the destruction behavior implemented in abstract superclasses can access the 
	vtable of the concrete run-time type in C++. Using a message makes C++ 
	parallel the Smalltalk semantics for delete.  This manually calls the destructor."
	
	' this->~Heaper();' translateOnly.! !

!PtrArray methodsFor: 'smalltalk: conversion'!

asOrderedCollection
	| result |
	result _ OrderedCollection new: self count.
	0 almostTo: self count do: [ :i |
		(self fetch: i) ~~ NULL ifTrue:
			[result add: (self fetch: i)]].
	^result! !

!PtrArray methodsFor: 'smalltalk: bulk accessing'!

{Int32} indexOfObject: value {IntegerVar | IEEE64 | Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	| result n |
	start < 0
		ifTrue: [result := self basicSize + start + 1.
			result < 0
				ifTrue: [Heaper BLAST: #IndexOutOfBounds]]
		ifFalse: [result := start + 1].
	result > self basicSize
		ifTrue: [self halt . ^ -1 "Heaper BLAST: #IndexOutOfBounds"].
	nth >= 0 ifTrue:
		[n := nth.
			[|o|
			o := self basicAt: result.
			((value == NULL and: [o == NULL]) or: [value isEqual: o]) ifTrue:
				[n := n - 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result + 1.
		result < self basicSize] whileTrue.
		^-1]
	ifFalse:
		[n := nth.
			[|o|
			o := self basicAt: result.
			((value == NULL and: [o == NULL]) or: [value isEqual: o]) ifTrue:
				[n := n + 1.
				n == 0
					ifTrue: [^result]].
			result := result - 1.
		result > 0] whileTrue.
		^-1]! !

!PtrArray methodsFor: 'bulk accessing'!

{Int32} indexOf: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}

	^self indexOfObject: value with: start with: n!

{Int32} indexOfEQ: value {Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	| result n |
	start < 0
		ifTrue: [result := self basicSize + start + 1.
			result < 0
				ifTrue: [Heaper BLAST: #IndexOutOfBounds]]
		ifFalse: [result := start + 1].
	result > self basicSize
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	nth >= 0 ifTrue:
		[n := nth.
			[|o|
			o := self basicAt: result.
			value == o ifTrue:
				[n := n - 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result + 1.
		result < self basicSize] whileTrue.
		^-1]
	ifFalse:
		[n := nth.
			[|o|
			o := self basicAt: result.
			value == o ifTrue:
				[n := n + 1.
				n == 0
					ifTrue: [^result]].
			result := result - 1.
		result > 0] whileTrue.
		^-1]!

{Int32} indexOfEQOrNull: value {Heaper | NULL}
	with: start {Int32 default: Int32Zero}
	with: nth {Int32 default: 1}
	"The index of the nth occurrence of anything but the given value at or after (before if n is negative) the given index, or -1 if there is none."

	| result n |
	start < 0
		ifTrue: [result := self basicSize + start + 1.
			result < 0
				ifTrue: [Heaper BLAST: #IndexOutOfBounds]]
		ifFalse: [result := start + 1].
	result > self basicSize
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	nth >= 0 ifTrue:
		[n := nth.
			[|o|
			o := self basicAt: result.
			(value == o or: [o == NULL]) ifTrue:
				[n := n - 1.
				n == 0
					ifTrue: [^result - 1]].
			result := result + 1.
		result < self basicSize] whileTrue.
		^-1]
	ifFalse:
		[n := nth.
			[|o|
			o := self basicAt: result.
			(value == o or: [o == NULL]) ifTrue:
				[n := n + 1.
				n == 0
					ifTrue: [^result]].
			result := result - 1.
		result > 0] whileTrue.
		^-1]!

{Int32} indexPast: value {Heaper}
	with: start {Int32 default: Int32Zero}
	with: n {Int32 default: 1}

	^self indexPastObject: value with: start with: n!

{void} storeAll: value {Heaper default: NULL}
	with: count {Int32 default: -1}
	with: start {Int32 default: Int32Zero}

	| n {Int32} |
	n := self basicSize - start.
	count > n
		ifTrue: [Heaper BLAST: #IndexOutOfBounds].
	count >= 0
		ifTrue: [n := count].
	1 to: n do: [ :i |
		self basicAt: start + i put: value]! !

!PtrArray methodsFor: 'smalltalk: obsolescent'!

{Heaper vector} array
	self passe.
	^ self!

{void} zeroAll
	1 to: self size do: [ :i | self basicAt: i put: NULL].! !

!PtrArray methodsFor: 'smalltalk: fake arrays'!

{Heaper} at: idx {UInt32}
	^self basicAt: (idx + 1)!

{void} at: idx {UInt32} put: ptr {Heaper}
	self basicAt: (idx + 1) put: ptr! !

!PtrArray methodsFor: 'accessing'!

{void} at: index {Int32} store: pointer {Heaper | NULL}

	self basicAt: index + 1 put: pointer!

{void} at: index {Int32} storeValue: value {Heaper | NULL}
	"Store a value in a PrimArray, may be a Heaper, NULL, or a PrimValue.
	It is expected that most PrimArray clients will want to use less abstract
	access methods."

	self basicAt: index + 1 put: value!

{Heaper | NULL} fetch: index {Int32}
	"Retrieve a single element from the array. Does array bounds checking. Non-pointer arrays box up the contents in a PrimValue object."

	^self basicAt: 1 + index!

{Heaper | NULL} fetchValue: index {Int32}
	"Fetch a value from a PrimArray, may be a Heaper, NULL, or a PrimValue.
	Clients are expected to perform necessary casting.  It is expected that
	most PrimArray clients will want to use less abstract access methods."

	^self basicAt: 1 + index!

{Heaper} get: index {Int32}
	"Retrieve a single element from the array. Does array bounds checking. Non-pointer arrays box up the contents in a PrimValue object."
	| result |
	result _ self fetch: index.
	result == NULL ifTrue: [Heaper BLAST: #NullEntry].
	^ result!

{PrimSpec} spec

	^PrimSpec pointer! !

!PtrArray methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	self receiveIPtrArray: receiver.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	self sendPtrArray: xmtr.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PtrArray class
	instanceVariableNames: ''!

(PtrArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #(COPY xpp ); add: #ON.CLIENT; add: #MANUAL.RECIPE; add: #CONCRETE; yourself)!

!PtrArray class methodsFor: 'creation'!

{PtrArray} empty

	self thingToDo. "Cache a single array"
	^self create: Int32Zero!

{PtrArray CLIENT} nulls: count {Int32}

	^self create: count!

{PtrArray} wimpy: count {Int32}

	^PtrArray create: count! !

!PtrArray class methodsFor: 'smalltalk: self copy'!

create.Rcvr: rcvr {Rcvr}
	| count result |
	count _ rcvr receiveIntegerVar.
	result _ self create: count.
	0 to: count - 1 do: [ :i |
		result at: i store: (rcvr receiveHeaper)].
	^result! !

!PtrArray class methodsFor: 'smalltalk: smalltalk initialization'!

staticTimeNonInherited
	STArrayRecipe create: self with: (Smalltalk associationAt: #XppCuisine)! !

!PtrArray class methodsFor: 'smalltlak: passe'!

make: count {Int32}

	self passe. "use nulls:"!

make: count {UInt32} with: from {PtrArray}
	self passe.
	^ PtrArray create: count with: from! !PtrArray variableSubclass: #SharedPtrArray
	instanceVariableNames: 'myShareCount {Int4}'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-rparray'!

(SharedPtrArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #COPY; yourself)!

!SharedPtrArray methodsFor: 'accessing'!

{Int32 INLINE} shareCount
	^myShareCount!

{void INLINE} shareLess
	myShareCount _ myShareCount - 1!

{void INLINE} shareMore
	myShareCount _ myShareCount + 1!

{PrimSpec} spec

	^PrimSpec sharedPointer! !

!SharedPtrArray methodsFor: 'creation'!

create: count {Int32}

	super create: count.
	myShareCount := Int32Zero!

create: size {Int32}
	with: from {PrimIntegerArray}
	with: start {Int32}
	with: count {Int32}
	with: offset {Int32}
	
	super create: count with: from with: start with: count with: offset.
	myShareCount := Int32Zero! !

!SharedPtrArray methodsFor: 'generated:'!

create.Rcvr: receiver {Rcvr}
	super create.Rcvr: receiver.
	myShareCount _ receiver receiveHeaper.!

{void} sendSelfTo: xmtr {Xmtr}
	super sendSelfTo: xmtr.
	xmtr sendHeaper: myShareCount.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SharedPtrArray class
	instanceVariableNames: ''!

(SharedPtrArray getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #COPY; yourself)!

!SharedPtrArray class methodsFor: 'pseudo constructors'!

make: count {Int32}

	^self create: count! !

!SharedPtrArray class methodsFor: 'smalltlak: passe'!

make: count {Int32} with: from {PtrArray}
	self passe.
	^self create: count with: from with: 0 with: from count with: 0! !
(CxxSystemOrganization fileNamed: 'parray')
	comment: ''!


(CxxSystemOrganization getOrMakeFileNamed: 'parray')
	addClass: PrimArray
getOrMakeCxxClassDescription in: #public;
	addClass: PrimDataArray
getOrMakeCxxClassDescription in: #public;
	addClass: PrimFloatArray
getOrMakeCxxClassDescription in: #public;
	addClass: PrimIntegerArray
getOrMakeCxxClassDescription in: #public;
	addClass: IEEE64Array
getOrMakeCxxClassDescription in: #public;
	addClass: PrimIntArray
getOrMakeCxxClassDescription in: #public;
	addClass: IntegerVarArray
getOrMakeCxxClassDescription in: #public;
	addClass: Int32Array
getOrMakeCxxClassDescription in: #public;
	addClass: UInt32Array
getOrMakeCxxClassDescription in: #public;
	addClass: PtrArray
getOrMakeCxxClassDescription in: #public;
	addClass: SharedPtrArray
getOrMakeCxxClassDescription in: #public;
	addClass: IEEE32Array
getOrMakeCxxClassDescription in: #public;
	addClass: UInt8Array
getOrMakeCxxClassDescription in: #public!

CxxSystemOrganization tree: (CxxSystemOrganization tree addChild:
	((CxxTreeAssociation key: 'parray' value: #file) yourself))!