'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 8:59:31 pm'!

Recorder subclass: #BackfollowRecorder
	instanceVariableNames: '
		myPermissionsFilter {Filter of: ID}
		myEndorsementsFilter {Filter of: ID}
		myProp {Prop}'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-backrec'!

(BackfollowRecorder getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #OBSOLETE; yourself)!

!BackfollowRecorder methodsFor: 'create'!

create: wrapped {Manipulator} with: holder {Manipulator default: NULL}
	super create: wrapped with: holder.
	myEndorsementsFilter _ myPermissionsFilter _ IDSpace closedIDFilter.
	self restartBackfollowRecorder: NULL! !

!BackfollowRecorder methodsFor: 'recording'!

{void} filter: permissions {Filter of: ID} with: endorsements {Filter of: ID}
	myPermissionsFilter _ permissions.
	myEndorsementsFilter _ endorsements.
	myProp _ SensorProp make: permissions with: endorsements with: false!

{BooleanVar} passes: finder {PropFinder}
	"whether this recorder passes the finder's criteria
	constructs the joint from scratch to use the current permissions"
	^finder doesPass: (SensorPropJoint make: (self keyMaster cast: GateKeeper) permissionsFilter
		with: myEndorsementsFilter with: false)!

{void} record: stamp {Stamp}
	"record an indirect reference to the stamp.  As a convenience, the containing stamp is made readable.
	
	Suppresses redundant records of the same stamp.  (These are typically generated by a race between the now and future parts of a backfollow, which are guaranteed to err by overlapping rather than gapping.  They may also be generated by a crash/reboot during AgendaItem processing.)"
	
	| |
	
	"Look into the backfollow trail to see if stamp is already there.  If not
		Make a single-entry table
		containing the stamp we want to record.
		extract our keymaster
		make trivial containing edition from the table with our keymaster
			(for holding on to read permission on the stamp that we're recording
			 this will all be done in a completely different way after the no-clubs-on-stamps transformation.)
		Make a table 
		mapping from this Stamp's ID to the trivial containing stamp (to satisfy the interface for the next call
		and use it to inform that position in the backfollow trail with the containing stamp."
	
	(self foot orgl usedDomain hasMember: stamp iD) ifFalse:
		[ | table {MuTable} 
		    orgl {Orgl} 
		    agk {ActualGateKeeper} | 
		table _ IntegerTable make: 1.
		table atInt: IntegerVar0 store: stamp.
		agk _ self keyMaster cast: ActualGateKeeper.
		
		self knownBug. "publishes newly created singleton stamp to GateKeeper's userID.  Issue goes away with Works & Editions Semantics"
		orgl _ agk makeReadableOrgl: table.
		
		table _ MuTable make: IDSpace make.
		table at: stamp iD store: orgl stampID.
		self foot inform: table]!

{void} triggerIfMatchingStamp: stamp {Stamp} 
	with: finder {PropFinder} 
	with: fossil {Fossil of: BackfollowRecorder}
	"check whether this stamp is newly visible to this recorder and record it if so"
	
	"This test is to ensure that the semantics of delayed backfollow match that of
	the immediate backfollow -- i.e. the current stamp matches the endorsements filter,
	and there exists a stamp containing this one which passes the permissions filter"

	finder 
		cast: AbstractRecorderFinder into: [:arf |
			arf checkStamp: stamp with: self with: fossil]
		others: []! !

!BackfollowRecorder methodsFor: 'receiver'!

{void RECEIVE.HOOK} restartBackfollowRecorder: trans {Rcvr unused default: NULL}
	myProp _ SensorProp make: myEndorsementsFilter with: myPermissionsFilter with: false! !

!BackfollowRecorder methodsFor: 'accessing'!

{Filter} endorsementsFilter
	^myEndorsementsFilter!

{Filter} permissionsFilter
	^myPermissionsFilter!

{Prop} prop
	^myProp! !

!BackfollowRecorder methodsFor: 'protected: protected accessing'!

{BooleanVar} canCertify

	^super canCertify
		and: [self foot orgl domain isEqual: IDRegion allIDs]!

{Foot} foot
	^self wrapped cast: Foot! !

!BackfollowRecorder methodsFor: 'smalltalk: passe'!

{void} checkStamp: stamp {Stamp} with: finder {PropFinder}
	"check whether this stamp is newly visible to this recorder and record it if so"
	
	self passe!

{void} checkStamp: stamp {Stamp} 
	with: finder {PropFinder} 
	with: fossil {Fossil of: BackfollowRecorder}
	"check whether this stamp is newly visible to this recorder and record it if so"
	
	"This test is to ensure that the semantics of delayed backfollow match that of
	the immediate backfollow -- i.e. the current stamp matches the endorsements filter,
	and there exists a stamp containing this one which passes the permissions filter"

	self passe.	"Use triggerIfMatchingStamp:with:with:"! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BackfollowRecorder class
	instanceVariableNames: ''!

(BackfollowRecorder getOrMakeCxxClassDescription)
	attributes: ((Set new) add: #CONCRETE; add: #OBSOLETE; yourself)!

!BackfollowRecorder class methodsFor: 'pseudo constructors'!

make: gk {ActualGateKeeper} 
	with: permissions {Filter of: ID} 
	with: endorsements {Filter of: ID} 
	"Return a BackfollowRecorder onto a Foot onto a new partial stamp"
	| aFoot {Foot} result {BackfollowRecorder} |
	aFoot _ Foot grabNewWithFoot: gk with: IDRegion allIDs.
	result _ (XuServer currentServer
		wrap: (Sequence string: 'BackfollowRecorder') with: aFoot) cast: BackfollowRecorder.
	result filter: permissions with: endorsements.
	^result! !