'From Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989 on 22 March 1995 at 9:02:31 pm'!

CoordinateSpace subclass: #IDSpace
	instanceVariableNames: '
		myBackend {Sequence | NULL}
		mySpaceNumber {IntegerVar}
		myNewIDCounter {Counter}'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xanadu-Spaces-Unordered'!
IDSpace comment:
'A space of IDs, which can generate globally unique IDs.

Implementation note:
	myBackend - the identifier of the Server which generated this space. If NULL, then it was generated by the current Server (unless mySpaceNumber is -1, in which case it is the single global IDSpace shared by all Servers.
	mySpaceNumber - identifies which space this is. If -1, then it is the global ID space, and myBackend must be NULL.'!

(IDSpace getOrMakeCxxClassDescription)
	friends:
'friend IDSimpleStepper;
friend class BeGrandMap;
friend class IDTester;
friend class ID;
friend class IDRegion;';
	attributes: ((Set new) add: #PSEUDO.COPY; add: #CONCRETE; add: #ON.CLIENT; yourself)!

!IDSpace methodsFor: 'making'!

{IDRegion CLIENT} iDsFromServer: identifier {Sequence}
	"Essential. The Region of IDs in this space which might be genrated by the given Server"
	
	RaviNow shouldImplement.
	^NULL "fodder"!

{ID CLIENT} newID
	"Essential. A new ID guaranteed to be different from every other newID generated by this IDSpace or any IDSpace isEqual to it, on any Server. (Although of course IDs generated using this->oldID () may conflict if the right numbers happen to have been supplied.)"
	
	^ID make: self fetchIDSpace
		with: NULL
		with: myNewIDCounter increment!

{IDRegion CLIENT} newIDs: count {IntegerVar}
	"A region containing a finite number of globally unique IDs. See newID for uniqueness guarantees."
	
	^IDRegion make: self fetchIDSpace
			with: (IntegerRegion
				integerExtent: (myNewIDCounter incrementBy: count)
				with: count)
			with: NULL
			with: false! !

!IDSpace methodsFor: 'private: for friends'!

{Sequence} backend
	"Essential. The Server which created this IDSpace"
	
	myBackend == NULL ifTrue:
		[mySpaceNumber = -1
			ifTrue: [^Sequence zero]
			ifFalse: [^FeServer identifier]].
	^myBackend!

{Sequence | NULL} fetchBackend

	^myBackend!

{IDSpace | NULL} fetchIDSpace
	"NULL if this is the global IDSpace, self otherwise"
	
	(myBackend == NULL and: [mySpaceNumber = -1])
		ifTrue: [^NULL]
		ifFalse: [^self]!

{IDRegion} oldIDs: backend {Sequence} with: numbers {IntegerRegion}
	"Recreate a region of IDs from information that was stored outside the Server"
	
	backend isZero ifTrue:
		[(numbers intersects: (IntegerRegion after: IntegerVarZero))
			ifTrue: [Heaper BLAST: #InvalidRequest]
			ifFalse: [^IDRegion make: self fetchIDSpace
				with: numbers with: NULL with: false]]
	ifFalse:
		[ | table {MuTable} |
		(numbers isSubsetOf: (IntegerRegion after: IntegerVarZero))
			ifFalse: [Heaper BLAST: #InvalidRequest].
		(backend isEqual: FeServer identifier)
			ifTrue: [^IDRegion make: self fetchIDSpace
				with: numbers with: NULL with: false].
		table := MuTable make: SequenceSpace make.
		table at: backend store: numbers.
		^IDRegion make: self fetchIDSpace
				with: IntegerRegion make
				with: table asImmuTable
				with: false].
	^NULL "fodder"!

{IntegerVar} spaceNumber
	"Essential. Identifies this particular space among all those generated by the same Server."
	
	^mySpaceNumber! !

!IDSpace methodsFor: 'private: create'!

create: backend {Sequence | NULL} with: number {IntegerVar} with: counter {Counter}

	super create.
	myBackend := backend.
	mySpaceNumber := number.
	self finishCreation.
	myNewIDCounter := counter!

{void} finishCreation
	
	| myself {IDSpace} |
	(myBackend == NULL and: [mySpaceNumber = -1])
		ifTrue: [myself := NULL]
		ifFalse: [myself := self].
	self finishCreate: (IDRegion
			using: myself with: (IntegerSpace make emptyRegion cast: IntegerRegion) with: NULL with: false)
		with: (IDRegion
			using: myself with: (IntegerSpace make fullRegion cast: IntegerRegion) with: NULL with: true)
		with: (IDDsp make: self)
		with: (IDUpOrder make: self)
		with: NULL! !

!IDSpace methodsFor: 'testing'!

{UInt32} actualHashForEqual

	myBackend == NULL
		ifTrue: [^mySpaceNumber DOThashForEqual
			bitXor: self getCategory hashForEqual]
		ifFalse: [^(myBackend hashForEqual
			bitXor: mySpaceNumber DOThashForEqual)
			bitXor: self getCategory hashForEqual]!

{BooleanVar} isEqual: other {Heaper}

	other cast: IDSpace into: [ :space |
		^self == space
			or: [mySpaceNumber = space spaceNumber
				and: [(myBackend == NULL 
						and: [space fetchBackend == NULL])
					or: [myBackend ~~ NULL
						and: [space fetchBackend ~~ NULL
						and: [myBackend isEqual: space fetchBackend]]]]]]
	others:
		[^false].
	^false "fodder"! !

!IDSpace methodsFor: 'printing'!

{void} printOn: oo {ostream reference}

	oo << self getCategory name << '('.
	self fetchIDSpace == NULL
		ifTrue: [oo << '!!0']
		ifFalse: [oo << self backend << '.' << mySpaceNumber].
	oo << ')'! !

!IDSpace methodsFor: 'accessing'!

{UInt8Array CLIENT} export
	"Essential. Produce an array which can be handed to Server::importIDSpace on any Server to get back the same IDSpace"
	
	| xmtr {SpecialistXmtr} result {WriteVariableArrayStream} |
	result := WriteVariableArrayStream make: 200.
	xmtr := Binary2XcvrMaker make
		makeXmtr: (TransferSpecialist make: Cookbook make)
		with: result.
	ID exportSequence: xmtr with: self backend.
	xmtr sendIntegerVar: self spaceNumber.
	^result array! !

!IDSpace methodsFor: 'obsolete:'!

{Sequence} identifier
	"A Sequence uniquely identifying this IDSpace, so that
		FeServer::current ()->oldIDSpace (this->identifier ())
			->isEqual (this)"
	
	Ravi thingToDo. "get rid of this message and its clients"
	^self backend withLast: mySpaceNumber! !

!IDSpace methodsFor: 'hooks:'!

{void SEND.HOOK} sendIDSpaceTo: xmtr {Xmtr}
	xmtr sendHeaper: myBackend.
	xmtr sendIntegerVar: mySpaceNumber.
	xmtr sendHeaper: myNewIDCounter.! !

!IDSpace methodsFor: 'smalltalk: passe'!

{ID} oldID: identifier {Sequence}
	"Recreate an ID from its identifier."
	
	self passe.! !

!IDSpace methodsFor: 'generated:'!

{void} sendSelfTo: xmtr {Xmtr}
	
	self sendIDSpaceTo: xmtr.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IDSpace class
	instanceVariableNames: ''!

(IDSpace getOrMakeCxxClassDescription)
	friends:
'friend IDSimpleStepper;
friend class BeGrandMap;
friend class IDTester;
friend class ID;
friend class IDRegion;';
	attributes: ((Set new) add: #PSEUDO.COPY; add: #CONCRETE; add: #ON.CLIENT; yourself)!

!IDSpace class methodsFor: 'creation'!

{IDSpace CLIENT} global
	"Return the global ID space."
	
	^CurrentGrandMap fluidGet globalIDSpace!

{IDSpace CLIENT} import: data {PrimIntArray}
	"Essential. Take some information describing an IDSpace and create the IDSpace it was exported from."
	
	| rcvr {SpecialistRcvr} backend {Sequence} number {IntegerVar} |
	rcvr := Binary2XcvrMaker make
		makeRcvr: (TransferSpecialist make: Cookbook make)
		with: (XnReadStream make: (data cast: UInt8Array)).
	backend := ID importSequence: rcvr.
	number := rcvr receiveIntegerVar.
	^self make: backend with: number!

{IDSpace CLIENT} unique
	"Essential. Create a new globally unique space of IDs"
	
	^CurrentGrandMap fluidGet newIDSpace! !

!IDSpace class methodsFor: 'private: pseudo constructors'!

make: identifier {Sequence | NULL} with: number {IntegerVar}

	^self make: identifier with: number with: (CurrentGrandMap fluidGet
		getOrMakeIDCounter: identifier with: number)!

make: identifier {Sequence | NULL} with: number {IntegerVar} with: counter {Counter}

	| cgm {BeGrandMap} |
	cgm := CurrentGrandMap fluidFetch.
	(identifier ~~ NULL and: [identifier isZero
			or: [cgm ~~ NULL and: [identifier isEqual: cgm identifier]]])
		ifTrue: [^self create: NULL with: number with: counter].
	^self create: identifier with: number with: counter! !

!IDSpace class methodsFor: 'smalltalk: passe'!

{FilterSpace of: IDSpace} iDFilterSpace
	"The coordinate space of filters on IDRegions."
	self passe!

{Filter of: IDSpace} openIDFilter
	self passe.! !

!IDSpace class methodsFor: 'rcvr pseudo constructors'!

{Heaper} make.Rcvr: rcvr {Rcvr} 
	| memory {Heaper} backend {Sequence} space {IntegerVar} idCounter {Counter} |
	self thingToDo. "Should intern someday"
	memory _ (rcvr cast: SpecialistRcvr) makeIbid: IDSpace.
	backend _ rcvr receiveHeaper cast: Sequence.
	space _ rcvr receiveIntegerVar.
	idCounter _ rcvr receiveHeaper cast: Counter.
	^(IDSpace new.Become: memory)
		create: backend
		with: space
		with: idCounter! !

!IDSpace class methodsFor: 'smalltalk: system'!

info.stProtocol
"{UInt8Array CLIENT} export
{IDRegion CLIENT} iDsFromServer: identifier {Sequence}
{ID CLIENT} newID
{IDRegion CLIENT} newIDs: count {IntegerVar}
"! !