/* Copyright Xanadu Operating Company.  All Rights Reserved. */

/******************************************************************************
*                                                                            *
* The information contained herein is confidential, proprietary to Xanadu    *
* Operating Company, and considered a trade secret as defined in section     * 
* 499C of the penal code of the State of California.  Use of this information* 
* by anyone other than authorized employees of Xanadu is granted             *
* only under a  written non-disclosure agreement, expressly prescribing      * 
* the scope and  manner of such use.                                         *
*                                                                            *
***************************************************************************
Output from Objectworks for Smalltalk-80(tm), Version 2.5 of 29 July 1989
*/

#ifndef TRACEPX_CXX
#define TRACEPX_CXX


#ifndef CHOOSEX_HXX
#include "choosex.hxx"
#endif /* CHOOSEX_HXX */

#ifndef TRACEPX_HXX
#include "tracepx.hxx"
#endif /* TRACEPX_HXX */

#ifndef TRACEPX_IXX
#include "tracepx.ixx"
#endif /* TRACEPX_IXX */


#ifndef DISKMANX_HXX
#include "diskmanx.hxx"
#endif /* DISKMANX_HXX */

#ifndef PRIMTABX_HXX
#include "primtabx.hxx"
#endif /* PRIMTABX_HXX */

#ifndef SETX_HXX
#include "setx.hxx"
#endif /* SETX_HXX */




/* ************************************************************************ *
 * 
 *                    Class TracePosition 
 *
 * ************************************************************************ */


/* pseudo-constructors */


RPTR(TracePosition) TracePosition::make (APTR(BranchDescription) branch, UInt32 position){
	RETURN_CONSTRUCT(BoundedTrace,(branch, position));
}
/* testing */
/* accessing */

	/* automatic 0-argument constructor */
TracePosition::TracePosition() {}



/* ************************************************************************ *
 * 
 *                    Class   BoundedTrace 
 *
 * ************************************************************************ */


/* Trace Positions represent absolute coordinates in a tree-ordered space.  
The ent navigates to O-roots from H-roots in the northern direction using 
TracePositions.  Refer to the Ent document for a description of the current 
implementation.  TracePositions are generated by asking another tracePosition
for a newSuccesor.  The starting tracePosition is the initil TraceTree.  It 
corresponds to branch 1, position 1. */


/* testing */


UInt32 BoundedTrace::actualHashForEqual (){
	/* This isn't used now.  It might be later.  Hash 
		 based on both the branch and position  This hash
		 uses a couple of arbitrary primes. */
	
	/* 2^27-1 */
	return (myBranch->hashForEqual() + myPosition) * 10993 & 134217727;
}


BooleanVar BoundedTrace::isEqual (APTR(Heaper) another){
	BEGIN_CHOOSE(another) {
		BEGIN_KIND(BoundedTrace,bt) {
			{	BooleanVar crutch_Flag;
				/* myBranch->isEqual(bt->branch()) && myPosition == bt->position() */
				
				crutch_Flag = myBranch->isEqual(bt->branch());
				if(crutch_Flag) {
					crutch_Flag = myPosition == bt->position();
				}
				return crutch_Flag;
			}
		} END_KIND;
		BEGIN_OTHERS {
			return FALSE;
		} END_OTHERS;
	} END_CHOOSE;
	/* fodder */
	return FALSE;
}


BooleanVar BoundedTrace::isLE (APTR(TracePosition) another){
	/* Return true if the two positions are comparable and the 
	receiver is  
		less than the argument. The argument is put in the cache so that  
		many comparisons IN THE SAME ORDER will occur very fast. This  
		could probably check for both tracePositions in the cache for speed  
		even when the arguments are swapped. */
	
	return myBranch->doesInclude(myPosition, another);
}
/* accessing */


RPTR(BranchDescription) BoundedTrace::branch (){
	/* This should only be accessible in this module. */
	
	return (BranchDescription*) myBranch;
}


void BoundedTrace::cacheIn (APTR(PrimIndexTable) navCache){
	/* Cache the nav-data for the receiver in navCache. */
	
	IntegerVar oldValue;
	
	oldValue = navCache->fetch(myBranch);
	if (oldValue == -1) {
		navCache->store(myBranch, myPosition);
		myBranch->cacheRecur(navCache);
	} else {
		navCache->store(myBranch, max(oldValue, myPosition));
	}
}

void setbranch (BranchDescription *);

RPTR(TracePosition) BoundedTrace::newSuccessor (){
	/* Return a new successor to the receiver. The first 
	successor is on the same 
		branch with a higher position. Further successors are alllocated in a 
		binary-tree fashion along a new branch. */
	
	/* Known bug !!!! */
	
	/* This consistent count is upperbound over 12 runs of works test.
			It does not seem to be bound. */

	setbranch(this->myBranch);
BEGIN_CONSISTENT(14) {
		WPTR(TracePosition) 	returnValue;
		returnValue = myBranch->createAfter(this);
		return returnValue;
	} END_CONSISTENT;
}


RPTR(TracePosition) BoundedTrace::newSuccessorAfter (APTR(TracePosition) trace){
	/* Return a new tracePosition that is after both the receiver 
	and tracePos. */
	
	/* Known bug !!!! */
	
	/* This consistent count is upperbound over 12 runs of works test.
			It does not seem to be bound. */
	BEGIN_CONSISTENT(14) {
		SPTR(BranchDescription) branch;
		
		branch = myBranch->makeBranch(this, trace);
		myBranch->installBranchAfter(branch, this);
		CAST(BoundedTrace,trace)->branch()->installBranchAfter(branch, trace);
		WPTR(TracePosition) 	returnValue;
		returnValue = branch->nextPosition();
		return returnValue;
	} END_CONSISTENT;
}


UInt32 BoundedTrace::position (){
	/* This should only be accessible in this module. */
	
	return myPosition;
}


RPTR(ImmuSet) OF1(TracePosition) BoundedTrace::successors (){
	/* Return all the successors of the receiver. */
	
	WPTR(ImmuSet) OF1(TracePosition) 	returnValue;
	returnValue = myBranch->successorsOf(this);
	return returnValue;
}
/* create */


BoundedTrace::BoundedTrace (APTR(BranchDescription) br, UInt32 pos) {
	myBranch = br;
	myPosition = pos;
}

#ifndef TRACEPX_SXX
#include "tracepx.sxx"
#endif /* TRACEPX_SXX */



#endif /* TRACEPX_CXX */

